
unit bgrabitmap_042_u;

{$MODE objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, LMessages, LCLType,
  BGRABitmap, BGRABitmapTypes;

type
  { TForm1 }
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure WMEraseBkgnd(var message: TLMEraseBkgnd); message LM_ERASEBKGND;
  private
    { private declarations }
    ecran_virtuel,           { Écran virtuel.                                }
    dame,                    { Image de la dame.                             }
    sauvegarde: TBGRABitmap; { Image pour la restauration de l'arrière-plan. }
    fX, fY: integer;         { Position de la dame.                          }
  public
    { public declarations }
    procedure DessinDame(const aX, aY: integer);
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

const
  STEP = 5;

procedure TForm1.FormCreate(Sender: TObject);
var
  fond: TBGRABitmap;
begin
  { Création de l'écran virtuel. }
  ecran_virtuel := TBGRABitmap.Create(320, 40);
  
  { Création de l'image de la dame à partir du fichier. }
  dame := TBGRABitmap.Create('images\dame.bmp');
  
  { Remplacement du bleu par de la "couleur transparente". }
  dame.ReplaceColor(CSSMidnightBlue, BGRAPixelTransparent);
  
  { Création de l'image de fond à partir du fichier. }
  fond := TBGRABitmap.Create('images\ligne.bmp');
  
  { Dessin du fond sur l'écran virtuel. }
  ecran_virtuel.PutImage(0, 0, fond, dmSet);
  
  { Libération de l'image de fond. }
  fond.Free;
  
  { Dessin de la dame à sa position initiale. }
  DessinDame(0, 0);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ecran_virtuel.SaveToFile(ChangeFileExt(ParamStr(0), '.png'));
  ecran_virtuel.Free;
  dame.Free;
  sauvegarde.Free;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_LEFT : if fX >= 000 + STEP then DessinDame(fX - STEP, fY);
    VK_RIGHT: if fX <= 280 - STEP then DessinDame(fX + STEP, fY);
  end;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  { Dessin de l'écran virtuel dans la fenêtre }
  ecran_virtuel.Draw(Canvas, 0, 0);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  BorderStyle := bsSingle;
  ClientWidth := ecran_virtuel.Width;
  ClientHeight := ecran_virtuel.Height;
end;

procedure TForm1.WMEraseBkgnd(var message: TLMEraseBkgnd);
begin
  { Procédure créée et laissée vide à dessein, pour que le fond de la fenêtre ne soit pas effacé }
end;

procedure TForm1.DessinDame(const aX, aY: integer);
begin
  { Restauration du fond sauf au premier appel de la procédure }
  if Assigned(sauvegarde) then
    ecran_virtuel.PutImage(fX, fY, sauvegarde, dmSet);
  { Mise à jour de l'image de sauvegarde en vue de la prochaine restauration }
  BGRAReplace(sauvegarde, ecran_virtuel.GetPart(Rect(aX, aY, aX + 40, aY + 40)));
  { Dessin de la dame à sa nouvelle position }
  ecran_virtuel.PutImage(aX, aY, dame, dmDrawWithTransparency);
  { Enregistrement de la position }
  fX := aX;
  fY := aY;
  { Rafraîchissement de la fenêtre }
  Invalidate;
end;

end.
